#!/usr/bin/sage
#    Decomposing graphs into edges and triangles
#    
#    Daniel Kral  Bernard Lidicky  Taisa L. Martins  Yanitsa Pehova
#
# This is a program for generating the objective function.
# It lists all graphs on 7 vertices and for each graph $G$,
# it calculates coefficient:
#  2*|E(G)| - 3*fractional_triangle_packing(G)
#
# We are interested in covering edges by edge that costs 2
# or triangle, that costs 3. So for every edge, that is in
# a triangle, we get a discount one from the value.
# (Guess - it is used in reduced graph in regularity lemma,
#  so we need a fractional triangle packing and not just
#  deteting if an edge is in a triangle)
#

import itertools

# Convertning a graph to the format flag program can read later
#
def str_G(G):
    bstr = str(G.order())+" 0 "
    for i in range(G.order()):
        for j in range(i+1,G.order()):
            if G.has_edge(i,j):
                bstr += " 2"
            else:
                bstr += " 1"
        bstr += " "
    return bstr
    
# Computing fractional triangle packing
# using linear programming. Variables are for triangles,
# maximizing over sum of triangles subject to summing over
# any set of triangles that share an edge is at most one.
#
def fractional_triangle_packing(G):
    p = MixedIntegerLinearProgram(maximization=True, solver = "GLPK");
    t = p.new_variable(nonnegative = True); # triangles

    # Listing all triangles in lex order
    triangles=[]
    for x in G.vertices():
        for y in G.neighbors(x):
            if y <= x:
                continue
            for z in G.neighbors(y):
                if z <= x  or  z <= y:
                    continue
                if G.has_edge(x,z):
                    triangles.append([x,y,z])
                
    #print 'triangles =', triangles
    if (len(triangles) == 0):
        return 0
    
    # Constraint for every edge
    for j in G.edges(labels=False):
        # IDs of triangles on this edge
        in_triangles_id = []
        for tID in range(len(triangles)):
            if j[0] in triangles[tID] and  j[1] in triangles[tID]:
                in_triangles_id.append(tID)

        #print 'j=',j,' in_triangles_id=', in_triangles_id
                
        if len(in_triangles_id) > 0:
            p.add_constraint(1 >= sum(t[i] for i in in_triangles_id))
                
    p.set_objective(sum(t[tID] for tID in range(len(triangles)) ))

    return p.solve()

#G=Graph()
#G.add_edges([[0,1], [0,2], [1,3], [2,3]])
#G.show()
#print 2*G.size()-3*fractional_triangle_packing(G), str_G(G)

# Small note - objective function for Bernards program must NOT contain
# coefficient that is 0. So we skip 0.
#
#               /----- Number of vertices to try this on 
for G in graphs(7):
    coefficent =  2*(  2*G.size()-3*fractional_triangle_packing(G) )
    #       Extra 2* to make the objective function all integers
    if coefficent != 0:
        print coefficent, str_G(G)

